@@ -41,6 +41,7 @@
 #include <stdint.h>
 #include <ctype.h>
 #include <string.h>
+#include <zlib.h>
 #include "contigs.h"
 #include "mut.h"
 #include "mut_txt.h"
@@ -71,33 +72,33 @@ uint8_t nst_nt4_table[256] = {
     4, 4, 4, 4,  4, 4, 4, 4,  4, 4, 4, 4,  4, 4, 4, 4
 };
 
-#define __gen_read(x, start, iter) do {									\
-    for (i = (start), k = 0, ext_coor[x] = -10; i >= 0 && i < seq.l && k < s[x]; iter) {	\
-        mut_t c = currseq->s[i], mut_type = c & mutmsk;			\
-        if (ext_coor[x] < 0) {								\
+#define __gen_read(x, start, iter) do {                                    \
+    for (i = (start), k = 0, ext_coor[x] = -10; i >= 0 && i < seq.l && k < s[x]; iter) {    \
+        mut_t c = currseq->s[i], mut_type = c & mutmsk;            \
+        if (ext_coor[x] < 0) {                                \
             if (mut_type != NOCHANGE && mut_type != SUBSTITUTE) continue; \
-            ext_coor[x] = i;								\
+            ext_coor[x] = i;                                \
             if(1 == strand[x]) ext_coor[x] -= s[x]-1; \
-        }													\
+        }                                                    \
         if (mut_type == DELETE) { \
-            ++n_indel[x];				\
+            ++n_indel[x];                \
             if(1 == strand[x]) ext_coor[x]--; \
             if(0 == k) n_indel_first[x]++; \
         } \
         else if (mut_type == NOCHANGE || mut_type == SUBSTITUTE) { \
-            tmp_seq[x][k++] = c & 0xf;						\
+            tmp_seq[x][k++] = c & 0xf;                        \
             if (mut_type == SUBSTITUTE) { \
-                ++n_sub[x];			\
+                ++n_sub[x];            \
                 if(0 == k) n_sub_first[x]++; \
-            } 												\
-        } else {											\
-            mut_t n, ins;										\
+            }                                                 \
+        } else {                                            \
+            mut_t n, ins;                                        \
             assert(mut_type == INSERT); \
-            ++n_indel[x];									\
-            n_indel_first[x]++;							\
+            ++n_indel[x];                                    \
+            n_indel_first[x]++;                            \
             if(1 == mut_get_ins(currseq, i, &n, &ins)) { \
                 if(0 == strand[x]) { \
-                    if(k < s[x]) tmp_seq[x][k++] = c & 0xf;						\
+                    if(k < s[x]) tmp_seq[x][k++] = c & 0xf;                        \
                     while(n > 0 && k < s[x]) { \
                         tmp_seq[x][k++] = ins & 0x3;                \
                         --n, ins >>= 2; \
@@ -108,7 +109,7 @@ uint8_t nst_nt4_table[256] = {
                         tmp_seq[x][k++] = (ins >> ((n-1) << 1) & 0x3);                \
                         --n; \
                     } \
-                    if (k < s[x]) tmp_seq[x][k++] = c & 0xf;						\
+                    if (k < s[x]) tmp_seq[x][k++] = c & 0xf;                        \
                 } \
             } else { \
                 int32_t byte_index, bit_index; \
@@ -117,7 +118,7 @@ uint8_t nst_nt4_table[256] = {
                 insertion = mut_get_ins_long_n(currseq->ins[ins], &num_ins); \
                 if(0 == strand[x]) { \
                     byte_index = mut_packed_len(num_ins) - 1; bit_index = (num_ins-1) & 0x3 ; \
-                    if(k < s[x]) tmp_seq[x][k++] = c & 0xf;						\
+                    if(k < s[x]) tmp_seq[x][k++] = c & 0xf;                        \
                     while(num_ins > 0 && k < s[x]) { \
                         assert(0 <= byte_index); \
                         tmp_seq[x][k++] = (insertion[byte_index] >> (bit_index << 1)) & 0x3;                \
@@ -140,19 +141,18 @@ uint8_t nst_nt4_table[256] = {
                             byte_index++; \
                         } \
                     } \
-                    if (k < s[x]) tmp_seq[x][k++] = c & 0xf;						\
+                    if (k < s[x]) tmp_seq[x][k++] = c & 0xf;                        \
                 } \
-            }													\
+            }                                                    \
         } \
-    }														\
-    if (k != s[x]) ext_coor[x] = -10;						\
+    }                                                        \
+    if (k != s[x]) ext_coor[x] = -10;                        \
     if (1 == strand[x]) { \
         for (k = 0; k < s[x]; ++k) tmp_seq[x][k] = tmp_seq[x][k] < 4? 3 - tmp_seq[x][k] : 4; \
-    } 														\
+    }                                                         \
 } while (0)
 
 /* Simple normal random number generator, copied from genran.c */
-
 double ran_normal()
 { 
   static int iset = 0; 
@@ -174,6 +174,11 @@ double ran_normal()
   }
 }
 
+void unreachable(char *message) {
+    fprintf(stderr, "\n[dwgsim_core] Error: %s\n", message);
+    exit(1);
+}
+
 int32_t ran_num(double prob, int32_t n)
 {
   int32_t i, r;
@@ -532,8 +537,7 @@ void dwgsim_core(dwgsim_opt_t * opt)
               }
           }
       }
-
-      if(0 == opt->muts_only) {
+      else {
           if(0 == n_ref && opt->C < 0) {
               n_pairs = opt->N - n_sim;
           }
@@ -605,6 +609,7 @@ void dwgsim_core(dwgsim_opt_t * opt)
               continue;
           }
           else if (n_pairs < 0) { // NB: this should not happen
+              fprintf(stderr, "[dwgsim_core] #4 skip sequence '%s' as not enough pairs found\n", name);
               // not enough pairs
               continue;
           }
@@ -617,6 +622,7 @@ void dwgsim_core(dwgsim_opt_t * opt)
       mut_print(name, &seq, mutseq[0], mutseq[1], opt->fp_mut, opt->fp_vcf);
 
       if(0 == opt->muts_only) {
+          int num_failed = 0;
           for (ii = 0; ii != n_pairs; ++ii, ++ctr) { // the core loop
               if(0 == (ctr % 10000)) {
                   fprintf(stderr, "\r[dwgsim_core] %llu",
@@ -685,23 +691,27 @@ void dwgsim_core(dwgsim_opt_t * opt)
                   n_sub_first[0] = n_sub_first[1] = n_indel_first[0] = n_indel_first[1] = n_err_first[0] = n_err_first[1] = 0;
                   num_n[0]=num_n[1]=0;
 
-                  // strand
-                  if(2 == opt->strandedness || (0 == opt->strandedness && ILLUMINA == opt->data_type)) {
-                      // opposite strand by default for Illumina
-                      strand[0] = 0; strand[1] = 1; 
+                  // set read one's strand
+                  switch(opt->read_one_strand) {
+                      case 0: strand[0] = (drand48() < 0.5) ? 1 : 0; break;
+                      case 1: strand[0] = 0; break;
+                      case 2: strand[0] = 1; break;
+                      default: unreachable("read strand was not between 0-2");
                   }
-                  else if(1 == opt->strandedness || (0 == opt->strandedness && (SOLID == opt->data_type || IONTORRENT == opt->data_type))) {
-                      // same strands by default for SOLiD
-                      strand[0] = 0; strand[1] = 0; 
-                  }
-                  else {
-                      // should not reach here
-                      assert(1 == 0);
-                  }
-                  if (drand48() < 0.5) { // which strand ?
-                      // Flip strands 
-                      strand[0] = (1 + strand[0]) % 2;
-                      strand[1] = (1 + strand[1]) % 2;
+
+                  // set read two's strand
+                  switch(opt->strandedness) {
+                      case 0: // default to data type
+                          switch (opt->data_type) {
+                              case ILLUMINA: strand[1] = 1 - strand[0]; break; // paired end for Illumina (opposite strand)
+                              case SOLID: 
+                              case IONTORRENT: strand[1] = strand[0]; break; // mate pair for SOLiD and IonTorrent (same strand)
+                              default: unreachable("data type was not between 0-2");
+                          }
+                          break;
+                      case 1: strand[1] = strand[0]; break; // mate pair (same strand)
+                      case 2: strand[1] = 1 - strand[0]; break; // paired end (opposite strand)
+                      default: unreachable("strandedness was not between 0-2");
                   }
 
                   // generate the reads in base space
@@ -712,12 +722,8 @@ void dwgsim_core(dwgsim_opt_t * opt)
                                * 5' E2 -----> .... E1 -----> 3'
                                * 3'           ....           5'
                                */
-                              if(0 == opt->is_inner) {
-                                  __gen_read(0, pos + d - s[0], ++i); 
-                              }
-                              else {
-                                  __gen_read(0, pos + s[1] + d, ++i); 
-                              }
+                              int r0_pos = (0 == opt->is_inner) ? (pos + d - s[0]) : (pos + s[1] + d - 1);
+                              __gen_read(0, r0_pos, ++i); 
                               __gen_read(1, pos, ++i);
                           }
                           else { // - strand
@@ -725,13 +731,9 @@ void dwgsim_core(dwgsim_opt_t * opt)
                                * 3'           ....            5'
                                * 5' <----- E1 .... <----- E2  3'
                                */
-                              __gen_read(0, pos + s[0], --i);
-                              if(0 == opt->is_inner) {
-                                  __gen_read(1, pos + d, --i);
-                              }
-                              else {
-                                  __gen_read(1, pos + s[0] + d + s[1], --i);
-                              }
+                              int r1_pos = (0 == opt->is_inner) ? (pos + d - 1) : (pos + s[0] + d + s[1] - 1);
+                              __gen_read(0, pos + s[0] - 1, --i);
+                              __gen_read(1, r1_pos, --i);
                           }
                       }
                       else { // opposite strand
@@ -740,25 +742,17 @@ void dwgsim_core(dwgsim_opt_t * opt)
                                * 5' E1 -----> ....           3'
                                * 3'           .... <----- E2 5'
                                */
+                              int r1_pos = (0 == opt->is_inner) ? (pos + d - 1) : (pos + s[0] + d + s[1] - 1);
                               __gen_read(0, pos, ++i);
-                              if(0 == opt->is_inner) {
-                                  __gen_read(1, pos + d, --i);
-                              }
-                              else {
-                                  __gen_read(1, pos + s[0] + d + s[1], --i);
-                              }
+                              __gen_read(1, r1_pos, --i);
                           }
                           else { // - strand
                               /*
                                * 5' E2 -----> ....           3'
                                * 3'           .... <----- E1 5'
                                */
-                              if(0 == opt->is_inner) {
-                                  __gen_read(0, pos + d, --i);
-                              }
-                              else {
-                                  __gen_read(0, pos + s[1] + d + s[0], --i); 
-                              }
+                              int r0_pos = (0 == opt->is_inner) ? (pos + d - 1) : (pos + s[1] + d + s[0] - 1); 
+                              __gen_read(0, r0_pos, --i);
                               __gen_read(1, pos, i++);
                           }
                       }
@@ -785,8 +779,14 @@ void dwgsim_core(dwgsim_opt_t * opt)
                   if (ext_coor[0] < 0 || ext_coor[1] < 0 || opt->max_n < num_n[0] || opt->max_n < num_n[1]) { // fail to generate the read(s)
                       --ii;
                       --ctr;
+                      num_failed++;
+                      if (num_failed > 10000) {
+                          fprintf(stderr, "\r[dwgsim_core] failed to generate a read after %d trials\n", num_failed);
+                          exit(1);
+                      }
                       continue;
                   }
+                  num_failed = 0;
 
                   if(SOLID == opt->data_type) {
                       // Convert to color sequence, use the first base as the adaptor
@@ -858,61 +858,65 @@ void dwgsim_core(dwgsim_opt_t * opt)
                       qstr[i] = 0;
                       // BWA
                       FILE *fpo = (0 == j) ? opt->fp_bwa1: opt->fp_bwa2;
-                      if(ILLUMINA == opt->data_type || IONTORRENT == opt->data_type) {
-                          fprintf(fpo, "@%s%s%s_%u_%u_%1u_%1u_%1u_%1u_%d:%d:%d_%d:%d:%d_%llx/%d\n", 
-                                  (NULL == opt->read_prefix) ? "" : opt->read_prefix,
-                                  (NULL == opt->read_prefix) ? "" : "_",
-                                  name, ext_coor[0]+1, ext_coor[1]+1, strand[0], strand[1], 0, 0,
-                                  n_err[0], n_sub[0], n_indel[0],
-                                  n_err[1], n_sub[1],n_indel[1],
-                                  (unsigned long long)ii, j+1);
-                          for (i = 0; i < s[j]; ++i)
-                            fputc("ACGTN"[(int)tmp_seq[j][i]], fpo);
-                          fprintf(fpo, "\n+\n%s\n", qstr);
-                      }
-                      else {
-                          // Note: BWA ignores the adapter and the first color, so this is a misrepresentation 
-                          // in samtools.  We must first skip the first color.  Basically, a 50 color read is a 
-                          // 49 color read for BWA.
-                          //
-                          // Note: BWA outputs F3 to read1, annotated as read "2", and outputs R3 to read2,
-                          // annotated as read "1".
-                          fprintf(fpo, "@%s%s%s_%u_%u_%1u_%1u_%1u_%1u_%d:%d:%d_%d:%d:%d_%llx/%d\n", 
-                                  (NULL == opt->read_prefix) ? "" : opt->read_prefix,
-                                  (NULL == opt->read_prefix) ? "" : "_",
-                                  name, ext_coor[0]+1, ext_coor[1]+1, strand[0], strand[1], 0, 0,
-                                  n_err[0] - n_err_first[0], n_sub[0] - n_sub_first[0], n_indel[0] - n_indel_first[0], 
-                                  n_err[1] - n_err_first[1], n_sub[1] - n_sub_first[1], n_indel[1] - n_indel_first[1],
-                                  (unsigned long long)ii, 2 - j);
-                          //fputc('A', fpo);
-                          for (i = 1; i < s[j]; ++i)
-                            fputc("ACGTN"[(int)tmp_seq[j][i]], fpo);
-                          fprintf(fpo, "\n+\n");
-                          for (i = 1; i < s[j]; ++i) 
-                            fputc(qstr[i], fpo);
-                          fprintf(fpo, "\n");
+                      if (NULL != fpo) {
+                        if(ILLUMINA == opt->data_type || IONTORRENT == opt->data_type) {
+                            gzprintf(fpo, "@%s%s%s_%u_%u_%1u_%1u_%1u_%1u_%d:%d:%d_%d:%d:%d_%llx/%d\n", 
+                                    (NULL == opt->read_prefix) ? "" : opt->read_prefix,
+                                    (NULL == opt->read_prefix) ? "" : "_",
+                                    name, ext_coor[0]+1, ext_coor[1]+1, strand[0], strand[1], 0, 0,
+                                    n_err[0], n_sub[0], n_indel[0],
+                                    n_err[1], n_sub[1],n_indel[1],
+                                    (unsigned long long)ii, j+1);
+                            for (i = 0; i < s[j]; ++i)
+                              gzputc(fpo, "ACGTN"[(int)tmp_seq[j][i]]);
+                            gzprintf(fpo, "\n+\n%s\n", qstr);
+                        }
+                        else {
+                            // Note: BWA ignores the adapter and the first color, so this is a misrepresentation 
+                            // in samtools.  We must first skip the first color.  Basically, a 50 color read is a 
+                            // 49 color read for BWA.
+                            //
+                            // Note: BWA outputs F3 to read1, annotated as read "2", and outputs R3 to read2,
+                            // annotated as read "1".
+                            gzprintf(fpo, "@%s%s%s_%u_%u_%1u_%1u_%1u_%1u_%d:%d:%d_%d:%d:%d_%llx/%d\n", 
+                                    (NULL == opt->read_prefix) ? "" : opt->read_prefix,
+                                    (NULL == opt->read_prefix) ? "" : "_",
+                                    name, ext_coor[0]+1, ext_coor[1]+1, strand[0], strand[1], 0, 0,
+                                    n_err[0] - n_err_first[0], n_sub[0] - n_sub_first[0], n_indel[0] - n_indel_first[0], 
+                                    n_err[1] - n_err_first[1], n_sub[1] - n_sub_first[1], n_indel[1] - n_indel_first[1],
+                                    (unsigned long long)ii, 2 - j);
+                            //gzputc(fpo, 'A');
+                            for (i = 1; i < s[j]; ++i)
+                              gzputc(fpo, "ACGTN"[(int)tmp_seq[j][i]]);
+                            gzprintf(fpo, "\n+\n");
+                            for (i = 1; i < s[j]; ++i) 
+                              gzputc(fpo, qstr[i]);
+                            gzprintf(fpo, "\n");
+                        }
                       }
 
                       // BFAST output
-                      fprintf(opt->fp_bfast, "@%s%s%s_%u_%u_%1u_%1u_%1u_%1u_%d:%d:%d_%d:%d:%d_%llx\n", 
-                              (NULL == opt->read_prefix) ? "" : opt->read_prefix,
-                              (NULL == opt->read_prefix) ? "" : "_",
-                              name, ext_coor[0]+1, ext_coor[1]+1, strand[0], strand[1], 0, 0,
-                              n_err[0], n_sub[0], n_indel[0], n_err[1], n_sub[1], n_indel[1],
-                              (unsigned long long)ii);
-                      if(ILLUMINA == opt->data_type || IONTORRENT == opt->data_type) {
-                          for (i = 0; i < s[j]; ++i)
-                            fputc("ACGTN"[(int)tmp_seq[j][i]], opt->fp_bfast);
-                          fprintf(opt->fp_bfast, "\n+\n%s\n", qstr);
-                      }
-                      else {
-                          fputc('A', opt->fp_bfast);
-                          for (i = 0; i < s[j]; ++i)
-                            fputc("01234"[(int)tmp_seq[j][i]], opt->fp_bfast);
-                          fprintf(opt->fp_bfast, "\n+\n");
-                          for (i = 0; i < s[j]; ++i) 
-                            fputc(qstr[i], opt->fp_bfast);
-                          fprintf(opt->fp_bfast, "\n");
+                      if (NULL != opt->fp_bfast) {
+                          gzprintf(opt->fp_bfast, "@%s%s%s_%u_%u_%1u_%1u_%1u_%1u_%d:%d:%d_%d:%d:%d_%llx\n", 
+                                  (NULL == opt->read_prefix) ? "" : opt->read_prefix,
+                                  (NULL == opt->read_prefix) ? "" : "_",
+                                  name, ext_coor[0]+1, ext_coor[1]+1, strand[0], strand[1], 0, 0,
+                                  n_err[0], n_sub[0], n_indel[0], n_err[1], n_sub[1], n_indel[1],
+                                  (unsigned long long)ii);
+                          if(ILLUMINA == opt->data_type || IONTORRENT == opt->data_type) {
+                              for (i = 0; i < s[j]; ++i)
+                                gzputc(opt->fp_bfast, "ACGTN"[(int)tmp_seq[j][i]]);
+                              gzprintf(opt->fp_bfast, "\n+\n%s\n", qstr);
+                          }
+                          else {
+                              gzputc(opt->fp_bfast, 'A');
+                              for (i = 0; i < s[j]; ++i)
+                                gzputc(opt->fp_bfast, "01234"[(int)tmp_seq[j][i]]);
+                              gzprintf(opt->fp_bfast, "\n+\n");
+                              for (i = 0; i < s[j]; ++i) 
+                                gzputc(opt->fp_bfast, qstr[i]);
+                              gzprintf(opt->fp_bfast, "\n");
+                          }
                       }
                   }
               }
@@ -962,61 +966,65 @@ void dwgsim_core(dwgsim_opt_t * opt)
                       }
                       // BWA
                       FILE *fpo = (0 == j) ? opt->fp_bwa1: opt->fp_bwa2;
-                      if(ILLUMINA == opt->data_type || IONTORRENT == opt->data_type) {
-                          fprintf(fpo, "@%s%s%s_%u_%u_%1u_%1u_%1u_%1u_%d:%d:%d_%d:%d:%d_%llx/%d\n", 
-                                  (NULL == opt->read_prefix) ? "" : opt->read_prefix,
-                                  (NULL == opt->read_prefix) ? "" : "_",
-                                  "rand", 0, 0, 0, 0, 1, 1,
-                                  0, 0, 0, 0, 0, 0,
-                                  (unsigned long long)rand_ii,
-                                  j+1);
-                          for (i = 0; i < s[j]; ++i)
-                            fputc("ACGTN"[(int)tmp_seq[j][i]], fpo);
-                          fprintf(fpo, "\n+\n%s\n", qstr);
-                      }
-                      else {
-                          // Note: BWA ignores the adapter and the first color, so this is a misrepresentation 
-                          // in samtools.  We must first skip the first color.  Basically, a 50 color read is a 
-                          // 49 color read for BWA.
-                          //
-                          // Note: BWA outputs F3 to read1, annotated as read "2", and outputs R3 to read2,
-                          // annotated as read "1".
-                          fprintf(fpo, "@%s%s%s_%u_%u_%1u_%1u_%1u_%1u_%d:%d:%d_%d:%d:%d_%llx/%d\n", 
-                                  (NULL == opt->read_prefix) ? "" : opt->read_prefix,
-                                  (NULL == opt->read_prefix) ? "" : "_",
-                                  "rand", 0, 0, 0, 0, 1, 1,
-                                  0, 0, 0, 0, 0, 0,
-                                  (unsigned long long)rand_ii,
-                                  2 - j);
-                          //fputc('A', fpo);
-                          for (i = 1; i < s[j]; ++i)
-                            fputc("ACGTN"[(int)tmp_seq[j][i]], fpo);
-                          fprintf(fpo, "\n+\n");
-                          for (i = 1; i < s[j]; ++i) 
-                            fputc(qstr[i], fpo);
-                          fprintf(fpo, "\n");
+                      if (NULL != fpo) {
+                          if(ILLUMINA == opt->data_type || IONTORRENT == opt->data_type) {
+                              gzprintf(fpo, "@%s%s%s_%u_%u_%1u_%1u_%1u_%1u_%d:%d:%d_%d:%d:%d_%llx/%d\n", 
+                                      (NULL == opt->read_prefix) ? "" : opt->read_prefix,
+                                      (NULL == opt->read_prefix) ? "" : "_",
+                                      "rand", 0, 0, 0, 0, 1, 1,
+                                      0, 0, 0, 0, 0, 0,
+                                      (unsigned long long)rand_ii,
+                                      j+1);
+                              for (i = 0; i < s[j]; ++i)
+                                gzputc(fpo, "ACGTN"[(int)tmp_seq[j][i]]);
+                              gzprintf(fpo, "\n+\n%s\n", qstr);
+                          }
+                          else {
+                              // Note: BWA ignores the adapter and the first color, so this is a misrepresentation 
+                              // in samtools.  We must first skip the first color.  Basically, a 50 color read is a 
+                              // 49 color read for BWA.
+                              //
+                              // Note: BWA outputs F3 to read1, annotated as read "2", and outputs R3 to read2,
+                              // annotated as read "1".
+                              gzprintf(fpo, "@%s%s%s_%u_%u_%1u_%1u_%1u_%1u_%d:%d:%d_%d:%d:%d_%llx/%d\n", 
+                                      (NULL == opt->read_prefix) ? "" : opt->read_prefix,
+                                      (NULL == opt->read_prefix) ? "" : "_",
+                                      "rand", 0, 0, 0, 0, 1, 1,
+                                      0, 0, 0, 0, 0, 0,
+                                      (unsigned long long)rand_ii,
+                                      2 - j);
+                              //gzputc(fpo, 'A');
+                              for (i = 1; i < s[j]; ++i)
+                                gzputc(fpo, "ACGTN"[(int)tmp_seq[j][i]]);
+                              gzprintf(fpo, "\n+\n");
+                              for (i = 1; i < s[j]; ++i) 
+                                gzputc(fpo, qstr[i]);
+                              gzprintf(fpo, "\n");
+                          }
                       }
 
                       // BFAST output
-                      fprintf(opt->fp_bfast, "@%s%s%s_%u_%u_%1u_%1u_%1u_%1u_%d:%d:%d_%d:%d:%d_%llx\n", 
-                              (NULL == opt->read_prefix) ? "" : opt->read_prefix,
-                              (NULL == opt->read_prefix) ? "" : "_",
-                              "rand", 0, 0, 0, 0, 1, 1,
-                              0, 0, 0, 0, 0, 0,
-                              (unsigned long long)rand_ii);
-                      if(ILLUMINA == opt->data_type || IONTORRENT == opt->data_type) {
-                          for (i = 0; i < s[j]; ++i)
-                            fputc("ACGTN"[(int)tmp_seq[j][i]], opt->fp_bfast);
-                          fprintf(opt->fp_bfast, "\n+\n%s\n", qstr);
-                      }
-                      else {
-                          fputc('A', opt->fp_bfast);
-                          for (i = 0; i < s[j]; ++i)
-                            fputc("01234"[(int)tmp_seq[j][i]], opt->fp_bfast);
-                          fprintf(opt->fp_bfast, "\n+\n");
-                          for (i = 0; i < s[j]; ++i) 
-                            fputc(qstr[i], opt->fp_bfast);
-                          fprintf(opt->fp_bfast, "\n");
+                      if (NULL != opt->fp_bfast) {
+                          gzprintf(opt->fp_bfast, "@%s%s%s_%u_%u_%1u_%1u_%1u_%1u_%d:%d:%d_%d:%d:%d_%llx\n", 
+                                  (NULL == opt->read_prefix) ? "" : opt->read_prefix,
+                                  (NULL == opt->read_prefix) ? "" : "_",
+                                  "rand", 0, 0, 0, 0, 1, 1,
+                                  0, 0, 0, 0, 0, 0,
+                                  (unsigned long long)rand_ii);
+                          if(ILLUMINA == opt->data_type || IONTORRENT == opt->data_type) {
+                              for (i = 0; i < s[j]; ++i)
+                                gzputc(opt->fp_bfast, "ACGTN"[(int)tmp_seq[j][i]]);
+                              gzprintf(opt->fp_bfast, "\n+\n%s\n", qstr);
+                          }
+                          else {
+                              gzputc(opt->fp_bfast, 'A');
+                              for (i = 0; i < s[j]; ++i)
+                                gzputc(opt->fp_bfast, "01234"[(int)tmp_seq[j][i]]);
+                              gzprintf(opt->fp_bfast, "\n+\n");
+                              for (i = 0; i < s[j]; ++i) 
+                                gzputc(opt->fp_bfast, qstr[i]);
+                              gzprintf(opt->fp_bfast, "\n");
+                          }
                       }
                   }
                   rand_ii++;
@@ -1063,7 +1071,7 @@ int main(int argc, char *argv[])
   }
 
   // Open files
-  opt->fp_fa =	xopen(argv[optind+0], "r");
+  opt->fp_fa = xopen(argv[optind+0], "r");
   strcpy(fn_fai, argv[optind+0]); strcat(fn_fai, ".fai");
   opt->fp_fai = fopen(fn_fai, "r"); // NB: depends on returning NULL;
   strcpy(fn_tmp, argv[optind+1]); strcat(fn_tmp, ".mutations.txt");
@@ -1071,12 +1079,16 @@ int main(int argc, char *argv[])
   strcpy(fn_tmp, argv[optind+1]); strcat(fn_tmp, ".mutations.vcf");
   opt->fp_vcf = xopen(fn_tmp, "w");
   if(0 == opt->muts_only) {
-      strcpy(fn_tmp, argv[optind+1]); strcat(fn_tmp, ".bfast.fastq");
-      opt->fp_bfast = xopen(fn_tmp, "w");
-      strcpy(fn_tmp, argv[optind+1]); strcat(fn_tmp, ".bwa.read1.fastq");
-      opt->fp_bwa1 = xopen(fn_tmp, "w");
-      strcpy(fn_tmp, argv[optind+1]); strcat(fn_tmp, ".bwa.read2.fastq");
-      opt->fp_bwa2 = xopen(fn_tmp, "w");
+      if (opt->output_type != OUTPUT_TYPE_BWA) {
+          strcpy(fn_tmp, argv[optind+1]); strcat(fn_tmp, ".bfast.fastq.gz");
+          opt->fp_bfast = gzopen(fn_tmp, "w");
+      }
+      if (opt->output_type != OUTPUT_TYPE_BFAST) {
+          strcpy(fn_tmp, argv[optind+1]); strcat(fn_tmp, ".bwa.read1.fastq.gz");
+          opt->fp_bwa1 = gzopen(fn_tmp, "w");
+          strcpy(fn_tmp, argv[optind+1]); strcat(fn_tmp, ".bwa.read2.fastq.gz");
+          opt->fp_bwa2 = gzopen(fn_tmp, "w");
+      }
   }
 
   // Run simulation
@@ -1084,7 +1096,13 @@ int main(int argc, char *argv[])
 
   // Close files
   if(0 == opt->muts_only) {
-      fclose(opt->fp_fa); fclose(opt->fp_bfast); fclose(opt->fp_bwa1); fclose(opt->fp_bwa2); 
+      if (opt->output_type != OUTPUT_TYPE_BWA) {
+          gzclose(opt->fp_bfast); 
+      }
+      if (opt->output_type != OUTPUT_TYPE_BFAST) {
+          gzclose(opt->fp_bwa1); gzclose(opt->fp_bwa2); 
+      }
+      fclose(opt->fp_fa); 
   }
   if(NULL != opt->fp_fai) fclose(opt->fp_fai);
   fclose(opt->fp_mut);
