@@ -32,6 +32,7 @@
 #include <stdint.h>
 #include <ctype.h>
 #include <string.h>
+#include <zlib.h>
 #include "mut.h"
 #include "dwgsim.h"
 #include "dwgsim_opt.h"
@@ -56,6 +57,7 @@ dwgsim_opt_t* dwgsim_opt_init()
   opt->rand_read = 0.05;
   opt->data_type = ILLUMINA;
   opt->strandedness = 0;
+  opt->read_one_strand = 0;
   opt->max_n = 0;
   opt->flow_order = NULL;
   opt->flow_order_len = 0;
@@ -67,7 +69,8 @@ dwgsim_opt_t* dwgsim_opt_init()
   opt->fn_muts_input = NULL;
   opt->fn_muts_input_type = -1;
   opt->fn_regions_bed = NULL;
-  opt->fp_mut = opt->fp_bfast = opt->fp_bwa1 = opt->fp_bwa2 = NULL;
+  opt->fp_mut = NULL;
+  opt->fp_bfast = opt->fp_bwa1 = opt->fp_bwa2 = NULL;
   opt->fp_fa = opt->fp_fai = NULL;
   opt->read_prefix = NULL;
 
@@ -106,7 +109,7 @@ int dwgsim_opt_usage(dwgsim_opt_t *opt)
   fprintf(stderr, "         -r FLOAT      rate of mutations [%.4f]\n", opt->mut_rate);
   fprintf(stderr, "         -F FLOAT      frequency of given mutation to simulate low fequency somatic mutations [%.4f]\n", opt->mut_freq);
   fprintf(stderr, "                           NB: freqeuncy F refers to the first strand of mutation, therefore mutations \n"); 
-  fprintf(stderr, "                           on the second strand occour with a frequency of 1-F \n");
+  fprintf(stderr, "                           on the second strand occur with a frequency of 1-F \n");
   fprintf(stderr, "         -R FLOAT      fraction of mutations that are indels [%.2f]\n", opt->indel_frac);
   fprintf(stderr, "         -X FLOAT      probability an indel is extended [%.2f]\n", opt->indel_extend);
   fprintf(stderr, "         -I INT        the minimum length indel [%d]\n", opt->indel_min);
@@ -116,10 +119,14 @@ int dwgsim_opt_usage(dwgsim_opt_t *opt)
   fprintf(stderr, "                           0: Illumina\n");
   fprintf(stderr, "                           1: SOLiD\n");
   fprintf(stderr, "                           2: Ion Torrent\n");
-  fprintf(stderr, "         -S INT        generate reads [%d]:\n", opt->strandedness);
+  fprintf(stderr, "         -S INT        generate paired end reads with orientation [%d]:\n", opt->strandedness);
   fprintf(stderr, "                           0: default (opposite strand for Illumina, same strand for SOLiD/Ion Torrent)\n");
   fprintf(stderr, "                           1: same strand (mate pair)\n");
   fprintf(stderr, "                           2: opposite strand (paired end)\n");
+  fprintf(stderr, "         -A INT        generate paired end reads with read one [%d]:\n", opt->read_one_strand);
+  fprintf(stderr, "                           0: default (both, random)\n");
+  fprintf(stderr, "                           1: forward genomic strand\n");
+  fprintf(stderr, "                           2: reverse genomic strand\n");
   fprintf(stderr, "         -f STRING     the flow order for Ion Torrent data [%s]\n", (char*)opt->flow_order);
   fprintf(stderr, "         -B            use a per-base error rate for Ion Torrent data [%s]\n", __IS_TRUE(opt->use_base_error));
   fprintf(stderr, "         -H            haploid mode [%s]\n", __IS_TRUE(opt->is_hap));
@@ -133,6 +140,10 @@ int dwgsim_opt_usage(dwgsim_opt_t *opt)
   fprintf(stderr, "         -q STRING     a fixed base quality to apply (single character) [%s]\n", (NULL == opt->fixed_quality) ? "not using" : opt->fixed_quality);
   fprintf(stderr, "         -Q FLOAT      standard deviation of the base quality scores [%.2lf]\n", (NULL == opt->fixed_quality) ? opt->quality_std : 0.0);
   fprintf(stderr, "         -s INT        standard deviation of the distance for pairs [%.3f]\n", opt->std_dev);
+  fprintf(stderr, "         -o INT        output type for the FASTQ files [%d]:\n", opt->output_type);
+  fprintf(stderr, "                           0: interleaved (bfast) and per-read-end (bwa)\n");
+  fprintf(stderr, "                           1: per-read-end (bwa) only\n");
+  fprintf(stderr, "                           2: interleaved (bfast) only\n");
   fprintf(stderr, "         -h            print this message\n");
   fprintf(stderr, "\n");
   fprintf(stderr, "Note: For SOLiD mate pair reads and BFAST, the first read is F3 and the second is R3. For SOLiD mate pair reads\n");
@@ -192,7 +203,7 @@ dwgsim_opt_parse(dwgsim_opt_t *opt, int argc, char *argv[])
   int c;
   int muts_input_type = 0;
   
-  while ((c = getopt(argc, argv, "id:s:N:C:1:2:e:E:r:F:R:X:I:c:S:n:y:BHf:z:Mm:b:v:x:P:q:Q:h")) >= 0) {
+  while ((c = getopt(argc, argv, "id:s:N:C:1:2:e:E:r:F:R:X:I:c:S:A:n:y:BHf:z:Mm:b:v:x:P:q:Q:o:h")) >= 0) {
       switch (c) {
         case 'i': opt->is_inner = 1; break;
         case 'd': opt->dist = dwgsim_atoi(optarg, 'd', 0); break;
@@ -210,6 +221,7 @@ dwgsim_opt_parse(dwgsim_opt_t *opt, int argc, char *argv[])
         case 'I': opt->indel_min = dwgsim_atoi(optarg, 'I', 0); break;
         case 'c': opt->data_type = dwgsim_atoi(optarg, 'c', 0); break;
         case 'S': opt->strandedness = dwgsim_atoi(optarg, 'S', 0); break;
+        case 'A': opt->read_one_strand = dwgsim_atoi(optarg, 'A', 0); break;
         case 'n': opt->max_n = dwgsim_atoi(optarg, 'n', 0); break;
         case 'y': opt->rand_read = atof(optarg); break;
         case 'f': 
@@ -228,6 +240,7 @@ dwgsim_opt_parse(dwgsim_opt_t *opt, int argc, char *argv[])
         case 'P': free(opt->read_prefix); opt->read_prefix = strdup(optarg); break;
         case 'q': opt->fixed_quality = strdup(optarg); break;
         case 'Q': opt->quality_std = atof(optarg); break;
+        case 'o': opt->output_type = atoi(optarg); break;
         default: fprintf(stderr, "Unrecognized option: -%c\n", c); return 0;
       }
   }
@@ -277,6 +290,7 @@ dwgsim_opt_parse(dwgsim_opt_t *opt, int argc, char *argv[])
   __check_option(opt->indel_min, 1, INT32_MAX, "-I");
   __check_option(opt->data_type, 0, 2, "-c");
   __check_option(opt->strandedness, 0, 2, "-S");
+  __check_option(opt->read_one_strand, 0, 2, "-A");
   __check_option(opt->max_n, 0, INT32_MAX, "-n");
   __check_option(opt->rand_read, 0, 1.0, "-y");
   if(IONTORRENT == opt->data_type && NULL == opt->flow_order) {
@@ -296,17 +310,7 @@ dwgsim_opt_parse(dwgsim_opt_t *opt, int argc, char *argv[])
       fprintf(stderr, "Warning: remember to use the -P option with dwgsim_eval\n");
   }
 
-  if(0 < opt->length[1]){ //paired end / mate pair
-    int s = opt->length[0] + opt->length[1] + opt->is_inner;
-    double r = (s-opt->dist)*1.0/opt->std_dev;
-    if(r>6){
-      fprintf(stderr, "Error: command line option -d is too small for the read length (%d)\n",opt->dist);
-      return 0;
-    }
-    if(r>4){
-      fprintf(stderr, "Warning: command line option -d is small for the read length (%d). Generation speed could be affected.\n",opt->dist);
-    }
-  }
+  __check_option(opt->output_type, 0, 2, "-o");
 
   switch(muts_input_type) {
     case 0x0:
